<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:exslt="http://exslt.org/common"
  xmlns:func="http://exslt.org/func"
>

<xsl:output
        method="text"
        indent="no"
        encoding="ISO-8859-1"/>

<xsl:param name="tpl"/>

<!-- xsl:key name="tpl" match="document('frenchmg.tpl.xml')/templates/template" use="@name" -->

<xsl:strip-space elements="*"/>

<xsl:template match="class">
  <xsl:variable name="class">'<xsl:value-of select="@name"/>'</xsl:variable>
class(<xsl:value-of select="$class"/>).
<xsl:apply-templates>
  <xsl:with-param name="class" select="$class"/>
</xsl:apply-templates>
</xsl:template>

<xsl:template match="supers/s"> <!-- deprecated -->
  <xsl:param name="class"/>
super(<xsl:value-of select="$class"/>,'<xsl:value-of select="@name"/>').
</xsl:template>

<xsl:template match="super">
  <xsl:param name="class"/>
super(<xsl:value-of select="$class"/>,'<xsl:value-of select="@name"/>').
</xsl:template>


<xsl:template match="pf"> <!-- deprecated -->
  <xsl:param name="class"/>
<xsl:value-of select="name(..)"/>(<xsl:value-of select="$class"/>,'<xsl:value-of select="@name"/>').
</xsl:template>

<xsl:template match="provides[@name]|needs[@name]">
  <xsl:param name="class"/>
<xsl:value-of select="name(.)"/>(<xsl:value-of select="$class"/>,'<xsl:value-of select="@name"/>').
</xsl:template>



<xsl:template match="description/fs">
  <xsl:param name="class"/>
description(<xsl:value-of select="$class"/>,
            [<xsl:apply-templates/>]
).
</xsl:template>

<xsl:template match="node">
  <xsl:param name="class"/>
  <xsl:variable name="node">'<xsl:value-of select="@name"/>'</xsl:variable>
  <xsl:if test="fs/f[@name='cat']">
node(<xsl:value-of select="$class"/>,<xsl:value-of select="$node"/>).
  </xsl:if>
  <xsl:if test="fs/f[@name = 'optional']">
optional(<xsl:value-of select="$class"/>,<xsl:value-of select="$node"/>).
  </xsl:if>
  <xsl:apply-templates>
    <xsl:with-param name="class" select="$class"/>
    <xsl:with-param name="node" select="$node"/>
  </xsl:apply-templates>
</xsl:template>

<xsl:template match="node/fs">
  <xsl:param name="class"/>
  <xsl:param name="node"/>
nodefeature(<xsl:value-of select="$class"/>,
            <xsl:value-of select="$node"/>,
            [<xsl:apply-templates/>]
).
</xsl:template>

<!--
<xsl:template match="node/fs/f[@name = 'optional']">
</xsl:template>
-->

<xsl:template match="father|precedes|equals|dominates">
  <xsl:param name="class"/>
  <xsl:value-of select="name(.)"/>(<xsl:value-of select="$class"/>,'<xsl:value-of select="preceding-sibling::arg1[1]/@name"/>','<xsl:value-of select="following-sibling::arg2[1]/@name"/>').
</xsl:template>

<xsl:template match="relation">
  <xsl:param name="class"/>
  <xsl:value-of select="@rel"/>(<xsl:value-of select="$class"/>,'<xsl:value-of select="@arg1"/>','<xsl:value-of select="@arg2"/>').
</xsl:template>

<xsl:template match="atom[@val]"> <!-- deprecated -->
  <xsl:choose>
    <xsl:when test="@val=''"><xsl:text>_</xsl:text>
    </xsl:when>
    <xsl:when test="number(@val)"><xsl:value-of select="@val"/>
    </xsl:when>
    <xsl:otherwise>'<xsl:value-of select="@val"/>'</xsl:otherwise>
  </xsl:choose>
  <xsl:if test="not(position()=last())">,</xsl:if>
</xsl:template>

<xsl:template match="sym[@value]">
  <xsl:choose>
    <xsl:when test="number(@value)"><xsl:value-of select="@value"/>
    </xsl:when>
    <xsl:otherwise>'<xsl:value-of select="@value"/>'</xsl:otherwise>
  </xsl:choose>
  <xsl:if test="not(position()=last())">,</xsl:if>
</xsl:template>

<xsl:template match="atom[@val='empty']"> <!-- deprecated -->
  <xsl:text>-</xsl:text>
  <xsl:if test="not(position()=last())">,</xsl:if>
</xsl:template>

<xsl:template match="fs">[<xsl:apply-templates/>]
</xsl:template>

<xsl:template match="plus">
  <xsl:text>+</xsl:text>
  <xsl:if test="not(position()=last())">,</xsl:if>
</xsl:template>

<xsl:template match="minus">
  <xsl:text>-</xsl:text>
  <xsl:if test="not(position()=last())">,</xsl:if>
</xsl:template>

<xsl:template match="f[*]">'<xsl:value-of select="@name"/>' : <xsl:apply-templates/>
<xsl:if test="not(position()=last())">,</xsl:if>
</xsl:template>

<!-- warning: be careful of extra newlines because of rigid taggify.pl -->
<xsl:template match="f[not(*)]">'<xsl:value-of select="@name"/>'<xsl:text> : _</xsl:text>
<xsl:if test="not(position()=last())">,</xsl:if>
</xsl:template>

<xsl:template match="atom[starts-with(@val,'@')]"> <!-- deprecated -->
  <xsl:variable name="name" select="@val"/>
  [ %% macro = <xsl:value-of select="$name"/>
  <xsl:apply-templates select="document($tpl)/templates/template[@name = $name]"/>  
]
  <xsl:if test="not(position()=last())">,</xsl:if>
</xsl:template>

<xsl:template match="sym[starts-with(@value,'@')]"> 
  <xsl:variable name="name" select="@value"/>
  [ %% macro = <xsl:value-of select="$name"/>
  <xsl:apply-templates select="document($tpl)/templates/template[@name = $name]"/>  
]
  <xsl:if test="not(position()=last())">,</xsl:if>
</xsl:template>

<xsl:template match="var[@name and not(*)]">var('<xsl:value-of select="@name"/>')
</xsl:template>

<xsl:template match="var[@name and *]">var('<xsl:value-of select="@name"/>')^<xsl:apply-templates/>
</xsl:template>

<xsl:template match="vAlt">disj([<xsl:apply-templates/>])
</xsl:template>

<xsl:template match="not/vAlt">notdisj([<xsl:apply-templates/>])
</xsl:template>

<xsl:template match="not[not(vAlt)]">notdisj([<xsl:apply-templates/>])
</xsl:template>

<xsl:template match="equation">
<xsl:param name="class"/>
equation(
  <xsl:value-of select="$class"/>,
  <xsl:apply-templates select="*[1]" mode="equation"/>,
  <xsl:apply-templates select="*[2]" mode="equation"/>
).
</xsl:template>

<xsl:template match="equation/node|equation/var|equation/father" mode="equation">
  <xsl:value-of select="name()"/>('<xsl:value-of select="@name"/>') <xsl:apply-templates mode="path"/>
</xsl:template>

<xsl:template match="equation/description" mode="equation">
 desc <xsl:apply-templates mode="path"/>
</xsl:template>

<xsl:template match="f" mode="path">
  : '<xsl:value-of select="@name"/>' <xsl:apply-templates mode="path"/>
</xsl:template>

<xsl:template match="equation/value" mode="equation">
  value(<xsl:apply-templates/>)
</xsl:template>

</xsl:stylesheet>
